@@ -54,6 +54,7 @@
 #include <exchange-hierarchy.h>
 
 #include "exchange-component.h"
+#include <e2k-utils.h>
 
 #ifndef O_BINARY
 #define O_BINARY 0
@@ -84,6 +85,9 @@ static GObjectClass *parent_class = NULL;
 
 #define d(x)
 
+static icaltimezone *
+internal_get_timezone (ECalBackend *backend, const char *tzid);
+
 static ECalBackendSyncStatus
 is_read_only (ECalBackendSync *backend, EDataCal *cal, gboolean *read_only)
 {
@@ -515,6 +519,12 @@ remove_calendar (ECalBackendSync *backend, EDataCal *cal)
 	const char *uri;
 
 	d(printf("ecbe_remove_calendar(%p, %p)\n", backend, cal));
+
+	/* If we do not have a folder, then that means there is no corresponding folder on the server,
+	   thus pretend we removed it successfully. It's not there anyway, thus should be fine. */
+	if (!cbex->folder)
+		return GNOME_Evolution_Calendar_Success;
+
 	uri = e_folder_exchange_get_internal_uri (cbex->folder);
 	result = exchange_account_remove_folder (cbex->account, uri);
 	if (result == EXCHANGE_ACCOUNT_FOLDER_OK)
@@ -548,11 +558,73 @@ e_cal_backend_exchange_cache_sync_start (ECalBackendExchange *cbex)
 	g_hash_table_foreach (cbex->priv->objects, add_to_unseen, cbex);
 }
 
+static gboolean
+find_instance (ECalBackendExchange *cbex, ECalBackendExchangeComponent *ecomp, const char *rid, const char *lastmod)
+{
+	GList *l;
+	gboolean found = FALSE;
+
+	if (!ecomp->instances)	
+		return FALSE;
+
+	for (l = ecomp->instances; l != NULL; l = l->next) {
+		ECalComponent *comp = e_cal_component_new ();
+		ECalComponentRange recur_id;
+		struct icaltimetype inst_rid, new_rid;
+		time_t rtime;
+		icaltimezone *f_zone;
+
+		e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (l->data));
+		e_cal_component_get_recurid (comp, &recur_id);
+
+		rtime = e2k_parse_timestamp (rid);
+		new_rid = icaltime_from_timet (rtime, FALSE);
+
+		f_zone = internal_get_timezone ((ECalBackend *) cbex, recur_id.datetime.tzid);
+		recur_id.datetime.value->zone = f_zone;
+		inst_rid = icaltime_convert_to_zone (*recur_id.datetime.value, icaltimezone_get_utc_timezone ());
+
+		e_cal_component_free_datetime (&recur_id.datetime);
+		g_object_unref (comp);
+		
+		if (icaltime_compare (inst_rid, new_rid) == 0) {
+			found = TRUE;
+			break;
+		}
+	}
+	return found;
+}
+
+/**
+ * e_cal_backend_exchange_ensure_utc_zone:
+ * Makes sure the given icaltimetype is in UTC, because
+ * RFC 2445 says CREATED/DTSTAMP/LAST-MODIFIED always in UTC.
+ * @param cb ECalbackendExchage descendant.
+ * @param itt What to convert to UTC, if required.
+ **/
+void
+e_cal_backend_exchange_ensure_utc_zone (ECalBackend *cb, struct icaltimetype *itt)
+{
+	g_return_if_fail (cb != NULL);
+	g_return_if_fail (itt != NULL);
+
+	/* RFC 2445 - CREATED/DTSTAMP/LAST-MODIFIED always in UTC */
+	if (!icaltime_is_null_time (*itt) && !icaltime_is_utc (*itt)) {
+		if (!itt->zone)
+			icaltime_set_timezone (itt, e_cal_backend_internal_get_default_timezone (cb));
+
+		icaltimezone_convert_time (itt, (icaltimezone*) icaltime_get_timezone (*itt), icaltimezone_get_utc_timezone ());
+		icaltime_set_timezone (itt, icaltimezone_get_utc_timezone ());
+	}
+}
+
 gboolean
 e_cal_backend_exchange_in_cache (ECalBackendExchange *cbex,
 				 const char          *uid,
 				 const char          *lastmod,
-				 const char	     *href)
+				 const char	     *href,
+				 const char	     *rid
+				 )
 {
 	ECalBackendExchangeComponent *ecomp;
 
@@ -563,6 +635,9 @@ e_cal_backend_exchange_in_cache (ECalBackendExchange *cbex,
 		return FALSE;
 	g_hash_table_remove (cbex->priv->cache_unseen, ecomp->uid);
 
+	if (rid) 
+		return find_instance (cbex, ecomp, rid, lastmod);
+
 	if (strcmp (ecomp->lastmod, lastmod) < 0) {
 		g_hash_table_remove (cbex->priv->objects, uid);
 		return FALSE;
@@ -1830,7 +1905,6 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 	char *from_name = NULL, *from_email = NULL;
 	GSList *attach_list = NULL, *l, *new_attach_list = NULL;
 	char *fname, *file_contents = NULL, *filename, *dest_url, *mime_filename, *attach_file;
-	char *mime_type;
 	int len = 0;
 
 	if (!g_ascii_strcasecmp(e_cal_backend_exchange_get_owner_email (E_CAL_BACKEND_SYNC (cbex)), exchange_account_get_email_id (cbex->account)))
@@ -2019,6 +2093,9 @@ internal_get_timezone (ECalBackend *backend, const char *tzid)
 {
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
 
+	if (!tzid)
+		return NULL;
+
 	return g_hash_table_lookup (cbex->priv->timezones, tzid);
 }
 
